@@ -1,49 +1,50 @@
-// $Id: route-list.cc 1.7.1.1 Mon, 15 Dec 1997 14:43:04 -0800 cengiz $
-// 
+//  $Id: route-list.cc 1.5.1.10.1.16 Wed, 19 May 1999 17:03:35 -0700 wlee $
+//
 //  Copyright (c) 1994 by the University of Southern California
-//  and/or the International Business Machines Corporation.
 //  All rights reserved.
 //
-//  Permission to use, copy, modify, and distribute this software and
-//  its documentation in source and binary forms for lawful
-//  non-commercial purposes and without fee is hereby granted, provided
-//  that the above copyright notice appear in all copies and that both
-//  the copyright notice and this permission notice appear in supporting
-//  documentation, and that any documentation, advertising materials,
-//  and other materials related to such distribution and use acknowledge
-//  that the software was developed by the University of Southern
-//  California, Information Sciences Institute and/or the International
-//  Business Machines Corporation.  The name of the USC or IBM may not
-//  be used to endorse or promote products derived from this software
-//  without specific prior written permission.
+//  Permission to use, copy, modify, and distribute this software and its
+//  documentation in source and binary forms for lawful non-commercial
+//  purposes and without fee is hereby granted, provided that the above
+//  copyright notice appear in all copies and that both the copyright
+//  notice and this permission notice appear in supporting documentation,
+//  and that any documentation, advertising materials, and other materials
+//  related to such distribution and use acknowledge that the software was
+//  developed by the University of Southern California, Information
+//  Sciences Institute. The name of the USC may not be used to endorse or
+//  promote products derived from this software without specific prior
+//  written permission.
 //
-//  NEITHER THE UNIVERSITY OF SOUTHERN CALIFORNIA NOR INTERNATIONAL
-//  BUSINESS MACHINES CORPORATION MAKES ANY REPRESENTATIONS ABOUT
-//  THE SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS
-//  PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
-//  INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
-//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND 
-//  NON-INFRINGEMENT.
+//  THE UNIVERSITY OF SOUTHERN CALIFORNIA DOES NOT MAKE ANY
+//  REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
+//  PURPOSE.  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
+//  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
+//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE,
+//  TITLE, AND NON-INFRINGEMENT.
 //
-//  IN NO EVENT SHALL USC, IBM, OR ANY OTHER CONTRIBUTOR BE LIABLE FOR ANY
-//  SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES, WHETHER IN CONTRACT,
-//  TORT, OR OTHER FORM OF ACTION, ARISING OUT OF OR IN CONNECTION WITH,
-//  THE USE OR PERFORMANCE OF THIS SOFTWARE.
+//  IN NO EVENT SHALL USC, OR ANY OTHER CONTRIBUTOR BE LIABLE FOR ANY
+//  SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES, WHETHER IN CONTRACT, TORT,
+//  OR OTHER FORM OF ACTION, ARISING OUT OF OR IN CONNECTION WITH, THE USE
+//  OR PERFORMANCE OF THIS SOFTWARE.
 //
 //  Questions concerning this software should be directed to 
-//  info-ra@isi.edu.
+//  ratoolset@isi.edu.
 //
-//  Author(s): Cengiz Alaettinoglu <cengiz@isi.edu>
+//  Author(s): Cengiz Alaettinoglu <cengiz@ISI.EDU>
 
-#include "config.hh"
+
+#include "config.h"
 #include "route-list.hh"
-#include "dbase.hh"
 #include "object.hh"
 #include <fstream.h>
 #include "roe.hh"
-#include "rusage.hh"
+#include "util/rusage.hh"
+#include "irr/irr.hh"
+
+// Added by wlee@isi.edu
+extern IRR *whois;
+extern IRR *RLWhois;
 
-Whois RLWhois;
 
 char *RouteList::format(RouteList::Route *vr) {
    static char line[256];
@@ -52,7 +53,7 @@ char *RouteList::format(RouteList::Route *vr) {
    for (c = line; c < line + vr->indentation; ++c)
       *c = ' ';
 
-   Prefask_map(vr->route).get_address(line + vr->indentation);
+   vr->route.get_text(line + vr->indentation);
    for (c = line + strlen(line); c < line + 31; ++c)
       *c = ' ';
    *c = 0;
@@ -88,8 +89,8 @@ char *RouteList::format(RouteList::Route *vr) {
    strcat(line, "    ");
 
    RouteList::Route::db_as* p;
-   for (p = vr->db_as_l.head(); p; p = vr->db_as_l.next(p->db_as_l))
-      sprintf(line + strlen(line), "  %s:%s", p->db, p->as);
+   for (p = vr->db_as_l.head(); p; p = vr->db_as_l.next(p))
+      sprintf(line + strlen(line), "  %s:%-4d", p->db, p->as);
 
    return line;
 }
@@ -205,16 +206,16 @@ void RouteList::select(int index) {
 
    get_registration_now(vr);
 
-   for (p = vr->db_as_l.head(); p; p = vr->db_as_l.next(p->db_as_l)) {
-      tcl_Eval("button .buttonbar.a%s:%s -text {%s %s} -command { roe_show_object %s %s %s}", 
+   for (p = vr->db_as_l.head(); p; p = vr->db_as_l.next(p)) {
+      tcl_Eval("button .buttonbar.a%s:%d -text {%s %d} -command { roe_show_object %s %d %s}", 
 	       p->db, p->as, p->db, p->as, p->db, p->as, 
-	       Prefask_map(vr->route).get_address(buffer2));
-      tcl_Eval("pack .buttonbar.a%s:%s  -side left", p->db, p->as);
+	       vr->route.get_text(buffer2));
+      tcl_Eval("pack .buttonbar.a%s:%d  -side left", p->db, p->as);
    }
 
    p = vr->db_as_l.head();
    if (p)
-      object.display(p->db, p->as, Prefask_map(vr->route).get_address(buffer2));
+      object.display(p->db, p->as, vr->route.get_text(buffer2));
    else
       object.clear();
 }
@@ -226,9 +227,11 @@ void RouteList::get_registration_now(RouteList::Route *vr) {
    if ((vr->status & VR_IRR_QUERIED) || (vr->status & VR_IRR_DONT_QUERY))
       return;
 
-   whois.QueryKillResponse("!ufo=0");
-   whois.QueryResponse(response, 
-		       "!r%s,o", Prefask_map(vr->route).get_address(buffer));
+   // Modified by wlee@isi.edu
+   //   whois->QueryKillResponse("!ufo=0");
+   whois->setFullObjectFlag(false);
+   //   whois->QueryResponse(response, "!r%s,o", vr->route.get_text(buffer));
+   whois->getSourceOrigin(response, vr->route.get_text(buffer)); 
 
    if (!response) 
      {
@@ -245,6 +248,13 @@ void RouteList::get_registration_now(RouteList::Route *vr) {
    }
 }
 
+// response contains all the (database,AS#) pair having the route vr->route
+// For example, query sent to server:
+//   !ufo=0
+//   !r128.223.0.0/16,o
+// Response from the server
+//   MCI  AS3582
+//   RADB  AS3582
 void RouteList::parse_registration_response(char *response, 
 					    RouteList::Route *vr) {
    char dbase[64], *line, *next;
@@ -265,33 +275,25 @@ void RouteList::parse_registration_response(char *response,
 
       strcpy(dbase, strtok(line, " \t"));
       for (as = strtok(NULL, " \t"); as; as = strtok(NULL, " \t")) {
-	 p = new RouteList::Route::db_as(dbase, as);
-	 vr->db_as_l.append(p->db_as_l);
+	 p = new RouteList::Route::db_as(dbase, atoi(as + 2));
+	 vr->db_as_l.append(p);
       }      
    }
 
-   switch (vr->db_as_l.size()) {
-   case 0:
+   if (vr->db_as_l.isEmpty())
       vr->reg_status = VR_REGISTERED_NO;
-      break;
-   case 1:
+   else if (vr->db_as_l.isSingleton())
       vr->reg_status = VR_REGISTERED_SINGLY;
-      break;
-   default:
+   else
       vr->reg_status = VR_REGISTERED_MULTI;
-   }
 
    fp = vr->db_as_l.head();
    if (fp) {
       int multi_homed = 0;
       int homed_to_me = 0;
-      for (p = fp; 
-	   p; 
-	   p = vr->db_as_l.next(p->db_as_l)) {
-	 if (strcmp(fp->as, p->as))
-	    multi_homed = 1;
-	 if (!strcmp(p->as, routelist.as)) 
-	    homed_to_me = 1;
+      for (p = fp; p; p = vr->db_as_l.next(p)) {
+       	 if (p->as != fp->as)       multi_homed = 1;
+	 if (p->as == routelist.as) homed_to_me = 1;
       }
 	 
       if (multi_homed && homed_to_me)
@@ -316,29 +318,22 @@ RouteList::Route *RouteList::get_registration_response() {
 
    if (!routelist.vr_response_next)
       return NULL;
-
    for (vr = routelist.vr_response_next;
 	vr && (vr->status & VR_IRR_DONT_QUERY);
-	vr = routelist.rlist.next(vr->rlist))
+	vr = routelist.rlist.next(vr))
       ;      
 
    if (vr) {
-      for (p = routelist.rlist.next(vr->rlist);
+      for (p = routelist.rlist.next(vr);
 	   p && (p->status & VR_IRR_DONT_QUERY);
-	   p = routelist.rlist.next(p->rlist))
+	   p = routelist.rlist.next(p))
 	 ;      
       routelist.vr_response_next = p;
       
-      if (! RLWhois.Response(response))
+      //      if (! RLWhois->Response(response))
+      if (!RLWhois->getSourceOrigin(response))
 	 return vr;
-/*
-      char *p;
-      if (RLWhois.Response(p))
-	{
-	cout << p << endl;
-	delete p;dddd
-	}
-*/
+
       tcl_Eval(".statusbar.right configure -text {Pending Replies: %d}", 
 	       --irr_pending_replies);
 
@@ -358,17 +353,19 @@ RouteList::Route *RouteList::ask_registration() {
    if (!routelist.vr_query_next)
       return NULL;
 
+   // Added by wlee@isi.edu
+   //   if (!RLWhois.WriteReady()) return NULL;
+
    for (vr = routelist.vr_query_next;
 	vr && (vr->status & VR_IRR_DONT_QUERY);
-	vr = routelist.rlist.next(vr->rlist))
+	vr = routelist.rlist.next(vr))
       ;      
 
    if (vr) {
-      routelist.vr_query_next = routelist.rlist.next(vr->rlist);
+      routelist.vr_query_next = routelist.rlist.next(vr);
 
-      RLWhois.Query("!r%s,o", Prefask_map(vr->route).get_address(buffer));
-
-//      RLWhois.Query("!r%s", Prefask_map(vr->route).get_address(buffer));
+      //      RLWhois->Query("!r%s,o", vr->route.get_text(buffer));
+      RLWhois->querySourceOrigin(vr->route.get_text(buffer));
 
    } else 
       routelist.vr_query_next = NULL;
@@ -380,19 +377,21 @@ RouteList::Route *RouteList::ask_registration() {
 void RouteList::get_registrations(ClientData clientdata) {
    char buffer[1024];
    RouteList::Route *vr;
-
    if (! routelist.vr_response_next && ! routelist.vr_query_next)
       return;
 
-// ???
-//   if (! RLWhois.ReadReady()) {
-   if (! RLWhois.PendingData()) {
+   //   if (! RLWhois->PendingData()) {
+   if (!RLWhois->readReady()) {
+
       tcl_Eval("update");
       Tk_DoWhenIdle(get_registrations, (ClientData) NULL);
       return;
    }
 
-   ask_registration();
+// Modified by wlee
+// Send query before reading would improve the performance, but some servers
+// couldn't handle bursty queries.  Read before sending queries again would be the right way.
+//   ask_registration();
 
    if (vr = get_registration_response()) {
       if (vr->display_index != -1) {
@@ -405,7 +404,10 @@ void RouteList::get_registrations(ClientData clientdata) {
 	 routelist.select(0);
 
       tcl_Eval("update");
-   } 
+   }
+
+   // Moved by wlee
+   ask_registration();
 
    Tk_DoWhenIdle(get_registrations, (ClientData) NULL);
 }
@@ -416,13 +418,13 @@ void RouteList::get_registrations_init(RouteList::Route *first) {
    RouteList::Route *vr1 = (RouteList::Route *) 1;
    RouteList::Route *vr;
 
-   if (!RLWhois.is_open()) {
-// Modified by wlee since it's needless to Open() anymore
-//      RLWhois.Open(opt_hostname, opt_port, opt_database);
-      RLWhois.QueryKillResponse("!ufo=0");
-   }
+   // Modified by wlee@isi.edu
+   //   if (!RLWhois->is_open()) {
+   //      RLWhois->QueryKillResponse("!ufo=0");
+   //   }
+   RLWhois->setFullObjectFlag(false);
 
-   for (vr = first; vr; vr = rlist.next(vr->rlist))
+   for (vr = first; vr; vr = rlist.next(vr))
       if (! (vr->status & VR_IRR_DONT_QUERY))
 	 irr_pending_replies++;
 
@@ -432,6 +434,11 @@ void RouteList::get_registrations_init(RouteList::Route *first) {
    if (! vr_query_next && first) {
       vr_response_next = first;
       vr_query_next = first;
+// ??? A quick hack for ripe
+      if (IRR::GetProtocol() == IRR::ripe || IRR::GetProtocol() == IRR::bird)
+         vr1 = ask_registration();
+      else
+// Quick hack end here
       for (int i = 0; i < PENDING_REQ_WINDOW && vr1; ++i)
 	 vr1 = ask_registration();
 
@@ -442,10 +449,9 @@ void RouteList::get_registrations_init(RouteList::Route *first) {
 }
 
 void RouteList::init(void) {
-  // ???
   vrdisplayed = NULL;
   vrdisplayed_size = 0;
-  *as = 0;
+  as = 0;
   vr_query_next = NULL;
   vr_response_next = NULL;
   matching_reg = 0;
@@ -456,33 +462,46 @@ void RouteList::init(void) {
     delete irr_radix;
     irr_radix = NULL;
     }
-  registered_routes.clear();
   real_routes.clear();
-  rlist.unlink_all();
+  rlist.unlink();
   radix.clear();
   irr_pending_replies = 0; 
 }
 
-void RouteList::load(char *_as) {
-   strncpy(as, _as, sizeof(as));
+void RouteList::load(ASt _as) {
+  // Modified by wlee@isi.edu
+  as = _as;
+  //   strncpy(as, _as, sizeof(as));
 
-   Pix my_as;
-   my_as = AS_map.add_entry(as);
-   registered_routes = AS_map.expand(my_as);
+   const PrefixRanges *registered_routes = whois->expandAS(as);
+
+   // Added by wlee
+   if (registered_routes->isEmpty()) {
+     tcl_Eval("showWarning { No route for AS%d}", as);
+     return;
+   }
 
    RouteList::Route *vr;
+   for (int i = registered_routes->low(); 
+	i < registered_routes->fence();
+	i++)
+     radix.insert(new RouteList::Route((*registered_routes)[i]));
 
-   for (Pix i = registered_routes.first(); i; registered_routes.next(i)) {
-      vr = new RouteList::Route(registered_routes(i));
-      radix.insert(registered_routes(i), vr);
+   /*
+   for (PrefixRange *prefixRange = registered_routes.first(); 
+	prefixRange; 
+	prefixRange = registered_routes.next()) {
+      vr = new RouteList::Route(*prefixRange);
+      radix.insert(vr);
    }
+   */
 
    // below code is less eficient than above commented code
    // however user interface looks better
    // undo this once expand returns a RadixSet
    for (RadixNode *nd = &radix; nd; nd = nd->next())
       if (nd->state == RADIX_FULL)
-	 rlist.append(((Route *) nd->data)->rlist);
+	 rlist.append((Route *) nd->data);
 
    get_registrations_init(rlist.head());
 }
@@ -496,7 +515,7 @@ void RouteList::read_bgp_dump(char *fname) {
 
    tail = rlist.tail();
 
-   extern int readbgpdump(_SetOfPix&);
+   extern int readbgpdump(RadixSet &);
    extern FILE *bgpin;
    bgpin = fopen(fname, "r");
    if (! bgpin) {
@@ -506,69 +525,77 @@ void RouteList::read_bgp_dump(char *fname) {
    readbgpdump(real_routes);
    fclose(bgpin);
 
-   for (vr = rlist.head(); vr; vr = rlist.next(vr->rlist))
+   for (vr = rlist.head(); vr; vr = rlist.next(vr))
       if (real_routes.contains(vr->route))
-	 vr->rtd_status = VR_ROUTED_YES;
+         vr->rtd_status = VR_ROUTED_YES;
       else
 	 if (! (vr->rtd_status & VR_ROUTED_YES))
 	    vr->rtd_status = VR_ROUTED_NO;
 
-   define_irr_radix();
+   if (!define_irr_radix()) return;
 
-   for (Pix i = real_routes.first(); i; real_routes.next(i)) {
-      if (! radix.find(real_routes(i))){
-	 vr = new RouteList::Route(real_routes(i));
+   PrefixRange pr;
+   RadixSet::PrefixRangeIterator prIterator(&real_routes);
+   for (bool b = prIterator.first(pr); b; b = prIterator.next(pr)) {
+      if (! radix.find(&pr)){
+	 vr = new RouteList::Route(pr);
 	 vr->rtd_status = VR_ROUTED_YES;
 
-	 if (! irr_radix->find(vr->route)) {
+	 if (! irr_radix->find(&vr->route)) {
 	    vr->reg_status = VR_REGISTERED_NO;
 	    vr->status = VR_IRR_DONT_QUERY;
 	 }
 
-	 rlist.append(vr->rlist);
-	 radix.insert(real_routes(i), vr);
+	 rlist.append(vr);
+	 radix.insert(vr);
       }
    }
 
    display();
    
    if (tail)
-      get_registrations_init(rlist.next(tail->rlist));
+      get_registrations_init(rlist.next(tail));
 }
 
-void RouteList::define_irr_radix() {
+// Modified by wlee from void to bool
+bool RouteList::define_irr_radix() {
    //cerr << "measure_now before expand(IRR_ROUTES)" << endl;
    //cerr << ru << endl;
    if (! irr_radix) {
      tcl_Eval("popup_message {This will take 3 to 5 mins to finish, during which, your roe will freeze.  Please be patient!}");
 
-      Pix IRR_ROUTES = Community_map.add_entry("IRR-ROUTES");
-      _SetOfPix& irr_routes = Community_map.expand(IRR_ROUTES);
+     const PrefixRanges *irr_routes = whois->expandRSSet("RS-ANY");
 
-      //cerr << "measure_now after expand(IRR_ROUTES)" << endl;
-      //cerr << ru << endl;
+     // Added by wlee
+     if (!irr_routes) return false;
 
-      irr_radix = new Radix<Route>;
+     //cerr << "measure_now after expand(IRR_ROUTES)" << endl;
+     //cerr << ru << endl;
 
-      for (Pix p = irr_routes.first(); p; irr_routes.next(p))
-	 irr_radix->insert(irr_routes(p), NULL);
+     irr_radix = new Radix<Route>;
+
+     for (int i = irr_routes->low(); i < irr_routes->fence(); i++) {
+       irr_radix->insert(new RouteList::Route((*irr_routes)[i]));
+     }
 
    }
    //cerr << "measure_now after define_irr_radix()" << endl;
    //cerr << ru << endl;
+
+   return true;
 }
 
 void RouteList::get_more_specifics(char *selection) {
    RouteList::Route *vr;
    Route *tail;
 
-   define_irr_radix();
+   if (!define_irr_radix()) return;
 
    tail = rlist.tail();
 
    for (char *s = strtok(selection, " \t"); s; s = strtok(NULL, " \t")) {
       vr = vrdisplayed[atoi(s)];
-      RadixNode *rdnode = irr_radix->find(vr->route);
+      RadixNode *rdnode = irr_radix->find(&vr->route);
       if (rdnode) {
 	 get_more_specifics_add(rdnode->left);
 	 get_more_specifics_add(rdnode->rght);
@@ -581,7 +608,7 @@ void RouteList::get_more_specifics(char *selection) {
       display();
 
       if (tail)
-	 get_registrations_init(rlist.next(tail->rlist));
+	 get_registrations_init(rlist.next(tail));
    }
 }
 
@@ -591,17 +618,18 @@ void RouteList::get_more_specifics_add(RadixNode *rdnode) {
    if (!rdnode)
       return;
 
+   PrefixRange *prefixRange = &((RouteList::Route *)rdnode->data)->route;
+
    if (rdnode->state == RADIX_FULL) { // is it actually a route in IRR
-      RadixNode *node = radix.find(rdnode->route); 
+      RadixNode *node = radix.find(prefixRange); 
 
       if (!node || node->state == RADIX_EMPTY) { // is not in radix
-	 vr = new RouteList::Route(rdnode->route);
-	 rlist.append(vr->rlist);
+	 vr = new RouteList::Route(*prefixRange);
+	 rlist.append(vr);
 	 if (!node)
-	    radix.insert(vr->route, vr);
+	    radix.insert(vr);
 	 else {
 	    node->state |= RADIX_FULL;
-	    node->route = vr->route;
 	    node->data  = vr;
 	 }
       }
@@ -619,14 +647,14 @@ void RouteList::get_less_specifics(char *selection) {
    RadixNode *irr_node;
    RadixNode *node;
 
-   define_irr_radix();
+   if (!define_irr_radix()) return;
 
    tail = rlist.tail();
 
    for (char *s = strtok(selection, " \t"); s; s = strtok(NULL, " \t")) {
       vr = vrdisplayed[atoi(s)];
-      irr_node = irr_radix->find(vr->route);
-      node     = radix.find(vr->route);
+      irr_node = irr_radix->find(&vr->route);
+      node     = radix.find(&vr->route);
       if (irr_node && node) {
 	 // add the parents of irr_node
 	 for (irr_node = irr_node->prnt, node = node->prnt; 
@@ -635,10 +663,10 @@ void RouteList::get_less_specifics(char *selection) {
 	    if (irr_node->state == RADIX_FULL 
 		&& node->state == RADIX_EMPTY) { 
 	       // the node exists in irr_radix but not in radix, so add it
-	       vr = new RouteList::Route(irr_node->route);
-	       rlist.append(vr->rlist);
+	       PrefixRange *prefixRange = &((RouteList::Route *)irr_node->data)->route;
+	       vr = new RouteList::Route(*prefixRange);
+	       rlist.append(vr);
 	       node->state |= RADIX_FULL;
-	       node->route = vr->route;
 	       node->data  = vr;
 	    }
 	 }
@@ -649,7 +677,6 @@ void RouteList::get_less_specifics(char *selection) {
       display();
 
       if (tail)
-	 get_registrations_init(rlist.next(tail->rlist));
+	 get_registrations_init(rlist.next(tail));
    }
 }
-
